#include <string.h>

#ifndef _FBIP_
#define _FBIP_ 1

typedef char *STRING;

typedef double COORDINATE;

typedef COORDINATE COORDINATE_VECTOR[3];

#ifndef _BOOLEAN_DEFINED_
#define _BOOLEAN_DEFINED_ 1
typedef short BOOLEAN;
#endif

#define FALSE 0
#define TRUE 1

typedef struct fbip_struct {
  long blocks_wide;
  long blocks_hi;
  long n_blocks;
  long n_levels;
  double *z_levels;
  double xorg, yorg, zorg;
  double umin, umax, vmin, vmax;
  double du, dv;
  double ***block_projective_coeffs_array;
  double **height_interpolation_coeffs;
  long prev_block_ui;
  long prev_block_vi;
  double *inv_coeffs;
  int binaryp;
  double scale_factor; /* to convert units in long-lat world into degrees */
  char *ref_ellipsoid;
  int lat_long_p; 
  int binendianp;
  int doublep;
  double horiz_scale_factor; /* to convert horizontal units in cartesian 
                                 world into meters */
  double vert_scale_factor; /* to convert vertical units into meters */
  int debug;
} Fbip;

typedef Fbip *FBIP;


FBIP read_fbip_file();

void fbip_make_arrays();
void fbip_compute_height_interpolation_coeffs();
double ***allocate_corners_array();
void deallocate_corners_array();
void fill_fbip();

int 
fbip_project_in_block ();

COORDINATE *
fbip_project_to_view ();

int fbip_project_to_world();

/*void lprintf(char *format,  ...);
void debprintf(char *format,  ...);
void errprintf(char *format,  ...);*/

void lispprint();


/* originally in transforms.h */

typedef double *(*dvectfn)();

typedef dvectfn TRANSFORM_FN;

typedef struct coordinate_transform_struct {
  dvectfn transform_fn;
} Coordinate_Transform;

typedef Coordinate_Transform *COORDINATE_TRANSFORM;

typedef COORDINATE_TRANSFORM COORDINATE_PROJECTION;
 
COORDINATE *project_to_world();

#endif
